Package: RWNN
Type: Package
Title: Random Weight Neural Networks
Version: 0.4
Date: 2024-08-29
Authors@R: person(given = c("Søren", "B."), family = "Vilsen", role = c("aut", "cre"), email = "svilsen@math.aau.dk") 
Description: Creation, estimation, and prediction of random weight neural networks (RWNN), Schmidt et al. (1992) <doi:10.1109/ICPR.1992.201708>, including popular variants like extreme learning machines, Huang et al. (2006) <doi:10.1016/j.neucom.2005.12.126>, sparse RWNN, Zhang et al. (2019) <doi:10.1016/j.neunet.2019.01.007>, and deep RWNN, Henríquez et al. (2018) <doi:10.1109/IJCNN.2018.8489703>. It further allows for the creation of ensemble RWNNs like bagging RWNN, Sui et al. (2021) <doi:10.1109/ECCE47101.2021.9595113>, boosting RWNN, stacking RWNN, and ensemble deep RWNN, Shi et al. (2021) <doi:10.1016/j.patcog.2021.107978>.
License: MIT + file LICENSE
Imports: methods, quadprog, randtoolbox, Rcpp (>= 1.0.4.6), stats,
        utils
LinkingTo: Rcpp, RcppArmadillo
Encoding: UTF-8
RoxygenNote: 7.3.1
Suggests: tinytest
Depends: R (>= 4.1.0)
LazyData: true
NeedsCompilation: yes
Packaged: 2024-08-29 07:03:46 UTC; svilsen
Author: Søren B. Vilsen [aut, cre]
Maintainer: Søren B. Vilsen <svilsen@math.aau.dk>
Repository: CRAN
Date/Publication: 2024-09-03 14:50:06 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 01:51:59 UTC; unix
Archs: RWNN.so.dSYM
