# Generated by roxygen2: do not edit by hand

S3method(ae_rwnn,formula)
S3method(bag_rwnn,formula)
S3method(boost_rwnn,formula)
S3method(ed_rwnn,formula)
S3method(predict,ERWNN)
S3method(predict,RWNN)
S3method(reduce_network,ERWNN)
S3method(reduce_network,RWNN)
S3method(rwnn,formula)
S3method(stack_rwnn,formula)
export(ae_rwnn)
export(bag_rwnn)
export(boost_rwnn)
export(classify)
export(control_rwnn)
export(ed_rwnn)
export(reduce_network)
export(rwnn)
export(stack_rwnn)
importFrom(Rcpp,evalCpp)
importFrom(methods,formalArgs)
importFrom(methods,is)
importFrom(quadprog,solve.QP)
importFrom(randtoolbox,halton)
importFrom(randtoolbox,sobol)
importFrom(randtoolbox,torus)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,delete.response)
importFrom(stats,dnorm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(utils,methods)
useDynLib(RWNN)
